Crate undo [−] [src]
An undo-redo library with dynamic dispatch and automatic command merging. It uses the command pattern where the user modifies a receiver by applying commands on it.
The library has currently two data structures that can be used to modify the receiver:
- A stack that can push and pop commands to modify the receiver.
- A record that can roll the state of the receiver forwards and backwards.
Structs
| Commands |
Iterator over commands. |
| Error |
An error kind that holds the error and the command that caused the error. |
| Group |
A group of either stacks or records. |
| Record |
A record of commands. |
| RecordBuilder |
Builder for a record. |
| Stack |
A stack of commands. |
Traits
| Command |
Base functionality for all commands. |